package com.huiju.estateagents.eContract.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.bo.request.ContractRecordRequestBO;
import com.huiju.estateagents.common.CommonUtils;
import com.huiju.estateagents.eContract.entity.TaContract;
import com.huiju.estateagents.eContract.service.ITaContractService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 合同列表  前端控制器
 * </p>
 *
 * @author fxf
 * @since 2020-03-26
 */
@RestController
@RequestMapping("/api")
public class TaContractController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaContractController.class);

    @Autowired
    public ITaContractService iTaContractService;


    /**
     * 条件查询合同列表
     *
     * @param requestBO
     * @return
     */
    @RequestMapping(value = "/channel/contract/list", method = RequestMethod.GET)
    public ResponseBean listContractForChannel(ContractRecordRequestBO requestBO) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean = iTaContractService.listContractForChannel(requestBO);
            logger.info("TaContractController.listContractForChannel 返回结果:{}", JSONObject.toJSONString(responseBean));
        } catch (Exception e) {
            logger.error("TaContractController.listContractForChannel 异常：", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量结算前校验
     *
     * @return
     */
    @RequestMapping(value = "/channel/contract/checkBeforeBatchUpdatePrice", method = RequestMethod.GET)
    public ResponseBean checkBeforeBatchUpdatePrice(@RequestBody TaContract taContract) {
        ResponseBean responseBean = new ResponseBean();
        Integer fitNum = 0;
        Integer notFitNum = 0;
        Integer totalNum = taContract.getTaContractList().size();
        try {
            for (TaContract record : taContract.getTaContractList()) {
                TaContract contract = iTaContractService.getById(record.getContractId());
                if (contract == null) {
                    fitNum++;
                } else {
                    notFitNum++;
                }
            }

            JSONObject obj = new JSONObject();
            obj.put("fitNum", fitNum);
            obj.put("notFitNum", notFitNum);
            obj.put("totalNum", totalNum);
            responseBean.addSuccess(obj);
        } catch (Exception e) {
            logger.error("TaContractController.checkBeforeBatchUpdatePrice 异常：", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量结算
     *
     * @return
     */
    @RequestMapping(value = "/channel/contract/batchUpdatePrice", method = RequestMethod.PUT)
    public ResponseBean batchUpdateSettlementPrice(@RequestBody TaContract taContract) {
        ResponseBean responseBean = new ResponseBean();
        try {
            // 校验参数
            if (taContract.getSettlementPrice() == null) {
                responseBean.addError("请填写结算金额");
                return responseBean;
            }

            // 校验金额是否符合要求
            if (!CommonUtils.isAmount(taContract.getSettlementPrice() + "")) {
                responseBean.addError("结算金额不符合要求");
                return responseBean;
            }

            // 元转分
            Integer price = taContract.getSettlementPrice() * 100;
            taContract.getTaContractList().stream().forEach(record -> {
                record.setSettlementPrice(price);
            });
            responseBean.addSuccess(iTaContractService.updateBatchById(taContract.getTaContractList()));

        } catch (Exception e) {
            logger.error("TaContractController.batchUpdateSettlementPrice 异常：", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     *
     * @param taContract 实体对象
     * @return
     */
    @RequestMapping(value = "/taContract", method = RequestMethod.POST)
    public ResponseBean taContractAdd(@RequestBody TaContract taContract) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaContractService.save(taContract)) {
                responseBean.addSuccess(taContract);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taContractAdd -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     */
    @ResponseBody
    @RequestMapping(value = "/taContract/{id}", method = RequestMethod.DELETE)
    public ResponseBean taContractDelete(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaContractService.removeById(id)) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taContractDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     *
     * @param id         实体ID
     * @param taContract 实体对象
     * @return
     */
    @RequestMapping(value = "/taContract/{id}", method = RequestMethod.PUT)
    public ResponseBean taContractUpdate(@PathVariable Integer id,
                                         @RequestBody TaContract taContract) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaContractService.updateById(taContract)) {
                responseBean.addSuccess(taContract);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taContractUpdate -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/taContract/{id}", method = RequestMethod.GET)
    public ResponseBean taContractGet(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaContractService.getById(id));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taContractDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
